@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from trytond.model import fields
 from trytond.pyson import Eval, Bool
 from trytond.transaction import Transaction
@@ -12,11 +12,12 @@ __metaclass__ = PoolMeta
 class ShipmentOut:
     __name__ = 'stock.shipment.out'
     carrier = fields.Many2One('carrier', 'Carrier', states={
-            'readonly': Eval('state') != 'draft',
+            'readonly': ~Eval('state').in_(['draft', 'waiting', 'assigned',
+                    'packed']),
             },
         depends=['state'])
     cost_currency = fields.Many2One('currency.currency',
-            'Cost Currency', states={
+        'Cost Currency', states={
             'invisible': ~Eval('carrier'),
             'required': Bool(Eval('carrier')),
             'readonly': ~Eval('state').in_(['draft', 'waiting', 'assigned',
@@ -26,7 +27,7 @@ class ShipmentOut:
         'Cost Currency Digits'),
         'on_change_with_cost_currency_digits')
     cost = fields.Numeric('Cost',
-            digits=(16, Eval('cost_currency_digits', 2)), states={
+        digits=(16, Eval('cost_currency_digits', 2)), states={
             'invisible': ~Eval('carrier'),
             'readonly': ~Eval('state').in_(['draft', 'waiting', 'assigned',
                     'packed']),
@@ -56,25 +57,22 @@ class ShipmentOut:
 
     @fields.depends(methods=['inventory_moves'])
     def on_change_carrier(self):
-        return self.on_change_inventory_moves()
+        self.on_change_inventory_moves()
 
     @fields.depends('carrier', 'customer', 'inventory_moves')
     def on_change_inventory_moves(self):
-        Currency = Pool().get('currency.currency')
-
         try:
-            result = super(ShipmentOut, self).on_change_inventory_moves()
+            super(ShipmentOut, self).on_change_inventory_moves()
         except AttributeError:
-            result = {}
+            pass
         if not self.carrier:
-            return result
+            return
         with Transaction().set_context(self._get_carrier_context()):
             cost, currency_id = self.carrier.get_sale_price()
-        currency = Currency(currency_id)
-        result['cost'] = cost
-        result['cost_currency'] = currency.id
-        result['cost_currency_digits'] = currency.digits if currency else 2
-        return result
+        self.cost = cost
+        self.cost_currency = currency_id
+        self.cost_currency_digits = (self.cost_currency.digits
+            if self.cost_currency else 2)
 
     def _get_cost_tax_rule_pattern(self):
         'Get tax rule pattern for invoice line'
